@@ -92,7 +92,8 @@ EVENT_ALIASES = {
     "email": "ses",
     "entitlement.marketplace": "marketplace-entitlement-service",
     "es": "elasticsearch-service",
-    "events": "cloudwatch-events",
+    "events": "eventbridge",
+    "cloudwatch-events": "eventbridge",
     "iot-data": "iot-data-plane",
     "iot-jobs-data": "iot-jobs-data-plane",
     "iot1click-devices": "iot-1click-devices-service",
@@ -253,9 +254,16 @@ class _RetriesExceededError(Exception):
     pass
 
 
+class BadIMDSRequestError(Exception):
+    def __init__(self, request):
+        self.request = request
+
+
 class IMDSFetcher(object):
 
     _RETRIES_EXCEEDED_ERROR_CLS = _RetriesExceededError
+    _TOKEN_PATH = 'latest/api/token'
+    _TOKEN_TTL = '21600'
 
     def __init__(self, timeout=DEFAULT_METADATA_SERVICE_TIMEOUT,
                  num_attempts=1, base_url=METADATA_BASE_URL,
@@ -273,29 +281,56 @@ class IMDSFetcher(object):
             proxies=get_environ_proxies(self._base_url),
         )
 
-    def _get_request(self, url_path, retry_func):
+    def _fetch_metadata_token(self):
+        self._assert_enabled()
+        url = self._base_url + self._TOKEN_PATH
+        headers = {
+            'x-aws-ec2-metadata-token-ttl-seconds': self._TOKEN_TTL,
+        }
+        self._add_user_agent(headers)
+        request = botocore.awsrequest.AWSRequest(
+            method='PUT', url=url, headers=headers)
+        for i in range(self._num_attempts):
+            try:
+                response = self._session.send(request.prepare())
+                if response.status_code == 200:
+                    return response.text
+                elif response.status_code in (404, 403, 405):
+                    return None
+                elif response.status_code in (400,):
+                    raise BadIMDSRequestError(request)
+            except ReadTimeoutError:
+                return None
+            except RETRYABLE_HTTP_ERRORS as e:
+                logger.debug(
+                    "Caught retryable HTTP exception while making metadata "
+                    "service request to %s: %s", url, e, exc_info=True)
+        return None
+
+    def _get_request(self, url_path, retry_func, token=None):
         """Make a get request to the Instance Metadata Service.
 
         :type url_path: str
         :param url_path: The path component of the URL to make a get request.
-            This arg is appended to the base_url taht was provided in the
+            This arg is appended to the base_url that was provided in the
             initializer.
 
         :type retry_func: callable
         :param retry_func: A function that takes the response as an argument
              and determines if it needs to retry. By default empty and non
              200 OK responses are retried.
-         """
-        if self._disabled:
-            logger.debug("Access to EC2 metadata has been disabled.")
-            raise self._RETRIES_EXCEEDED_ERROR_CLS()
+
+        :type token: str
+        :param token: Metadata token to send along with GET requests to IMDS.
+        """
+        self._assert_enabled()
         if retry_func is None:
             retry_func = self._default_retry
         url = self._base_url + url_path
         headers = {}
-        if self._user_agent is not None:
-            headers['User-Agent'] = self._user_agent
-
+        if token is not None:
+            headers['x-aws-ec2-metadata-token'] = token
+        self._add_user_agent(headers)
         for i in range(self._num_attempts):
             try:
                 request = botocore.awsrequest.AWSRequest(
@@ -309,6 +344,15 @@ class IMDSFetcher(object):
                     "service request to %s: %s", url, e, exc_info=True)
         raise self._RETRIES_EXCEEDED_ERROR_CLS()
 
+    def _add_user_agent(self, headers):
+        if self._user_agent is not None:
+            headers['User-Agent'] = self._user_agent
+
+    def _assert_enabled(self):
+        if self._disabled:
+            logger.debug("Access to EC2 metadata has been disabled.")
+            raise self._RETRIES_EXCEEDED_ERROR_CLS()
+
     def _default_retry(self, response):
         return (
             self._is_non_ok_response(response) or
@@ -349,8 +393,9 @@ class InstanceMetadataFetcher(IMDSFetcher):
 
     def retrieve_iam_role_credentials(self):
         try:
-            role_name = self._get_iam_role()
-            credentials = self._get_credentials(role_name)
+            token = self._fetch_metadata_token()
+            role_name = self._get_iam_role(token)
+            credentials = self._get_credentials(role_name, token)
             if self._contains_all_credential_fields(credentials):
                 return {
                     'role_name': role_name,
@@ -376,18 +421,22 @@ class InstanceMetadataFetcher(IMDSFetcher):
             logger.debug("Max number of attempts exceeded (%s) when "
                          "attempting to retrieve data from metadata service.",
                          self._num_attempts)
+        except BadIMDSRequestError as e:
+            logger.debug("Bad IMDS request: %s", e.request)
         return {}
 
-    def _get_iam_role(self):
+    def _get_iam_role(self, token=None):
         return self._get_request(
             url_path=self._URL_PATH,
-            retry_func=self._needs_retry_for_role_name
+            retry_func=self._needs_retry_for_role_name,
+            token=token,
         ).text
 
-    def _get_credentials(self, role_name):
+    def _get_credentials(self, role_name, token=None):
         r = self._get_request(
             url_path=self._URL_PATH + role_name,
-            retry_func=self._needs_retry_for_credentials
+            retry_func=self._needs_retry_for_credentials,
+            token=token,
         )
         return json.loads(r.text)
 
@@ -869,9 +918,6 @@ def check_dns_name(bucket_name):
     if n < 3 or n > 63:
         # Wrong length
         return False
-    if n == 1:
-        if not bucket_name.isalnum():
-            return False
     match = LABEL_RE.match(bucket_name)
     if match is None or match.end() != len(bucket_name):
         return False
@@ -1373,3 +1419,13 @@ def get_encoding_from_headers(headers, default='ISO-8859-1'):
 
     if 'text' in content_type:
         return default
+
+
+class FileWebIdentityTokenLoader(object):
+    def __init__(self, web_identity_token_path, _open=open):
+        self._web_identity_token_path = web_identity_token_path
+        self._open = _open
+
+    def __call__(self):
+        with self._open(self._web_identity_token_path) as token_file:
+            return token_file.read()
